from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.contrib.auth.models import AbstractUser, auth
from .forms import StudentLecturesForm, SignUpForm, SubjectForm, Profile
from .models import StudentLectures, Subjects, User
from django.core.paginator import Paginator
from django.contrib.auth.decorators import login_required
# Create your views here.



@login_required(login_url='login')
def teachers_dashboard(request):
        '''
                Teachers dashboard 
        '''
        url = 'teacherapp/index.html'
        notice = StudentLectures.objects.all().order_by('-created_on')[:7]
        return render(request, url, {'notice':notice})

       

def login(request):
        '''
                Teachers dashboard
        '''
        url = 'teacherapp/login.html'

        # login request check if user is already logged in
        if request.user.is_active:
                if request.user.user_status == "Teacher":
                        return redirect('teacherapp:teachers_dashboard')
                elif request.user.user_status == "Student":
                        return redirect('studentapp:student_dashboard')

        if request.method == 'POST':
                username = request.POST['username']
                password = request.POST['password']

                user = auth.authenticate(username=username, password=password)

                if user is not None:
                        auth.login(request, user)
                        if request.user.user_status == "Teacher":
                                return redirect('teacherapp:teachers_dashboard')
                        elif request.user.user_status == "Student":
                                return redirect('studentapp:student_dashboard')
                else:
                        messages.info(request, 'Username or Password is wrong')
                        return redirect('login')
        else:
                return render(request, url)


def logout(request):
        username = request.user
        auth.logout(request)
        messages.info(request, f' {username}, logout successful.')
        return redirect('login')



@login_required(login_url='login')
def registration(request):
        '''
                       Registration Form
        '''
        url = 'teacherapp/registration_form.html'

        form = SignUpForm()
        if request.method == 'POST':
                form = SignUpForm(request.POST)
                username = request.POST.get('username')
                if form.is_valid():
                        form.save()
                        messages.success(request, f'{username} Information registerd successfully')
                        return redirect('teacherapp:registration')
                else:
                        
                        messages.warning(request, 'Error submitting registration form.')
                        return render(request, url, {'form': form})
        return render(request, url, {'form': form})



@login_required(login_url='login')
def profile(request):
        '''
                       Profile Page
        '''
        url = 'teacherapp/profile.html'

        data = {
                'username': request.user.username,
                'first_name': request.user.first_name,
                'last_name': request.user.last_name,
                'email': request.user.email,
        }

        form = Profile(initial=data)

        if request.method == 'POST':
                form = Profile(request.POST)

                if form.is_valid():
                        form.save()
                        messages.success(request, 'Student information registerd successfully')
                        return redirect('teacherapp:registration')
                else:
                        messages.warning(request, 'Error submitting Student Registration form.')
                        return redirect('teacherapp:registration')
        return render(request, url, {'form': form})


def reset_password(request):
        '''
                Reset Password
        :param request:
        :return:
        '''
        url = 'teacherapp/profile.html'

        try:
                if request.method == 'POST':
                        old_password = request.POST.get('old_password')
                        new_password = request.POST.get('new_password')
                        confirm_new_password = request.POST.get('confirm_new_password')

                        if (new_password == confirm_new_password) and request.user.check_password(old_password):
                                request.user.set_password(new_password)
                                request.user.save()
                                auth.logout(request)
                                messages.success(request, 'Successfully Password Updated')
                                return  redirect('login')
                        else:
                                messages.warning(request, 'Error changing password, try again')
                                return redirect('teacherapp:profile')

        except Exception as e:
                messages.warning(request, f'Error! {e}')
                return redirect('teacherapp:profile')
        finally:
                return redirect('teacherapp:profile')


@login_required(login_url='login')
def students(request):
        '''
                List Students in Dashboard
        '''
        url = 'teacherapp/students.html'
        try:
                students = User.objects.filter(user_status='Student').order_by('-date_joined')

                paginate = Paginator(students, 30)
                page_num = request.GET.get('page')
                students = paginate.get_page(page_num)

                if request.method == 'POST':
                        username = request.POST.get('search')
                        students =  User.objects.filter(username= username)
                        return render(request, url, {'students': students})
        except Exception as e:
                messages.warning(request, f'Error {e}')
                return redirect('teacherapp:students')
        finally:
                return render(request, url, {'students': students})

@login_required(login_url='login')
def teachers(request):
        '''
                List Teachers in Dashboard
        '''
        url = 'teacherapp/teachers.html'
        try:
                teachers = User.objects.filter(user_status='Teacher').order_by('-date_joined')

                paginate = Paginator(teachers, 30)
                page_num = request.GET.get('page')
                teachers = paginate.get_page(page_num)

                if request.method == 'POST':
                        username = request.POST.get('search')
                        teachers =  User.objects.filter(username= username)
                        return render(request, url, {'teachers': teachers})
        except Exception as e:
                messages.warning(request, f'Error {e}')
                return redirect('teacherapp:teachers')
        finally:
                return render(request, url, {'teachers': teachers})


@login_required(login_url='login')
def subjects_register(request):
        '''
                Subject Registrtaion View
        '''

        url = 'teacherapp/subject_register.html'
        form = SubjectForm()

        subjects = Subjects.objects.all()

        if request.method == 'POST':
                form = SubjectForm(request.POST)
                if form.is_valid():
                        data = form.save(commit=False)
                        data.user_register = User(id= request.user.id )
                        data.save()
                        messages.success(request, 'Successfully Subject Name Stored. ')
                        return redirect('teacherapp:subjects_register')
                else:
                        messages.warning(request, 'Error storing subject name')
                        return redirect('teacherapp:subjects_register')
        return render(request, url, {'form': form, 'subjects': subjects})

@login_required(login_url='login')
def lectures_register(request):
        '''
                Student Registration Form
        '''
        url = 'teacherapp/student_lecture.html'
        form = StudentLecturesForm()
        if request.method == 'POST':
                form = StudentLecturesForm(request.POST)
                if form.is_valid():
                        data = form.save(commit=False)
                        data.lecture_author = User(id=request.user.id)
                        data.save()
                        messages.success(request, 'lecture information saved successfully')
                        return redirect('teacherapp:lectures_register')
                else:
                        messages.warning(request, 'Error submitting lecture detail form.')
                        return redirect('teacherapp:lectures_register')
        return render(request, url, {'form': form})


@login_required(login_url='login')
def lectures_update(request, id):
        '''
                Student Registration Form
        '''
        url = 'teacherapp/student_lecture.html'
        l_id = int(id)
        lecture = get_object_or_404(StudentLectures, id=l_id)
        
        data = {
                'lecture_subject': lecture.lecture_subject,
                'lecture_title': lecture.lecture_title,
                'lecture_description': lecture.lecture_description,
                'lecture_url': lecture.lecture_url,
                'lecture_status': lecture.lecture_status
        }

        form = StudentLecturesForm(initial=data)

        if request.method == 'POST':
                form = StudentLecturesForm(request.POST, instance=lecture)
                if form.is_valid():
                        data = form.save(commit=False)
                        data.lecture_author = User(id=request.user.id)
                        data.save()
                        messages.success(request, 'lecture information Updated successfully')
                        return redirect('teacherapp:lectures_by_teacher')
                else:
                        messages.warning(request, 'Error submitting lecture detail form.')
                        return redirect('teacherapp:lectures_by_teacher')
        return render(request, url, {'form': form})


@login_required(login_url='login')
def lectures_list(request):
        '''
                Student Registration Form
        '''
        url = 'teacherapp/lectures.html'
        lectures = StudentLectures.objects.all().order_by('-created_on')

        paginate = Paginator(lectures, 10)
        page_num = request.GET.get('page')
        lectures = paginate.get_page(page_num)
        return render(request, url, {'lecture': lectures})


@login_required(login_url='login')
def lectures_by_staff(request):
        '''
                Student Registration Form
        '''
        url = 'teacherapp/lectures_teacher.html'
        lectures = StudentLectures.objects.filter(lecture_author=request.user.id).order_by('-created_on')
        paginate = Paginator(lectures, 10)
        page_num = request.GET.get('page')
        lectures = paginate.get_page(page_num)
        return render(request, url, {'lecture': lectures})